package org.tbasket.endpoint

import io.getquill.{Literal, NamingStrategy, SqliteDialect}
import io.getquill.context.qzio.{ZioContext, ZioJdbcContext}
import io.getquill.context.sql.idiom.SqlIdiom
import org.apache.logging.log4j.LogManager
import org.tbasket.auth.Authentificator
import org.tbasket.endpoint.Endpoint.LOG
import org.tbasket.handler.LoginHandler
import zio.*
import zio.http.*
import zio.http.ServerConfig.LeakDetectionLevel
import zio.http.model.Method.{GET, POST}
import zio.http.model.Status

import scala.collection.mutable
import zio.http.netty.client.ConnectionPool
import io.getquill.idiom.Idiom

import javax.sql.DataSource

class Endpoint(port: Int):
  
  
  // set generic required headers
  private def applyGenerics(response: Response): Response =
    response.withAccessControlAllowOrigin("*")
  
  private val app = Http.collectZIO[Request] {
    case r@POST -> _ / "login" =>
      LoginHandler.post(r)
    
    case r@method -> path =>
      val ipInsights = r.remoteAddress
        .map(ip => s": request received from $ip.")
        .getOrElse("")
      LOG.error(
        s"Was unable to find a handler for request '$path' with method $method ${ipInsights}"
      )
      ZIO.succeed(Response(Status.NotFound))
  }.map(applyGenerics)
  
  val run =
    val config = ServerConfig.default
      .port(port)
      .leakDetection(LeakDetectionLevel.PARANOID)
    
    val serverConfigLayer = ServerConfig.live(config)
    Server.install(app).flatMap { port =>
      LOG.info(s"Listening API entries on $port")
      ZIO.never
    }.provideSome[ZioJdbcContext[SqlIdiom, NamingStrategy] & Authentificator & DataSource](
      Scope.default,
      serverConfigLayer,
      ConnectionPool.fixed(4),
      ClientConfig.default,
      Server.live,
      Client.live
    )

object Endpoint:
  final val LOG = LogManager.getLogger("API")
